/*
 *  Licensed Materials - Property of IBM / HCL Technologies
 *  Restricted Material of IBM / HCL Technologies
 *  IBM Rational Test RealTime / HCL OneTest
 *  (c) Copyright IBM Corp. 2020 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *       |\      _,,,---,,_
 *       /,`.-'`'    -.  ;-;;,_
 *      |,4-  ) )-,_..;\ (  `'-'
 *     '---''(_/--'  `-'\_)
 *
 ********************************************
 */
/*
 * this file contains methods used to display model (in ascii art, to easy multiplatform execution)
 * it contains also all methods to display status (error, etc...) and some debug functions
 */

#include <stdio.h>
#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#endif
#include "core.h"
#include "display.h"

void displayHelp(void) {
	printf ("\nSample_maze [-help] [-seed=<seed>] [-x=<size>] [-y=<size>] [-noplay] \n");
	printf ("\t-help : display this message\n");
	printf ("\t-seed=<seed> : force random seed to allow replayability\n");
	printf ("\t-x=<size> : horizontal size (default = 30)\n");
	printf ("\t-y=<size> : vertical size (default = 25)\n");
	printf ("\t-noplay : no interactive play, just display maze and exit\n");
	printf ("\n");
}

void displayError(void) {
	printf ("invalid option\n");
}

void displayMatrix(void) {
	int i = 0;
	int j = 0;

	/* reverse order for displaying lines as it is stored in opposite order */
	if (model != NULL) {
		printf("\n\n   y\n");
		printf("   ^\n");
		for (j=size_y-1; j>=0; j--) {
			printf("%2u |",j);
			for (i=0; i<size_x; i++) {
				if (model[i][j] & WALL_LEFT) {
					printf("|");
				} else {
					printf(" ");
				}
				if (model[i][j] & WALL_DOWN) {
					printf("_");
				} else {
					printf(" ");
				}
				/* printf("%2u ",model[i][j]);*/
			}
			printf("\n");
		}
		printf("   +");
		for (i=0; i<size_x; i++) {
			printf("--");
		}
		printf("> x\n    ");
		for (i=0; i<size_x; i++) {
			printf("%2u",i);
		}
		printf("\n");
	}
}

void displayMatrixAndPos(int currentX, int currentY, int displayArrows, char c) {
	int i = 0;
	int j = 0;

	/* reverse order for displaying lines as it is stored in opposite order */
	if (model != NULL) {
		if (displayArrows) {
			printf("   y\n");
			printf("   ^\n");
		}
		for (j=size_y-1; j>=0; j--) {
			if (displayArrows) {
				printf("   | ");
			}
			for (i=0; i<size_x; i++) {
				if (model[i][j] & WALL_LEFT) {
					printf("|");
				} else {
					printf(" ");
				}
				if (((i == currentX) && (j == currentY)) || (model[i][j] & REPLAY_SPOT)){
					printf("%c%c",c,c);
				} else {
					printf("  ");
				}
			}
			printf("\n");
			if (displayArrows) {
				printf("%2u | ",j);
			}
			for (i=0; i<size_x; i++) {
				if (model[i][j] & WALL_LEFT) {
					printf("|");
				} else {
					printf(" ");
				}
				if (model[i][j] & WALL_DOWN) {
					printf("__");
				} else {
					printf("  ");
				}
			}

			printf("\n");
		}
		if (displayArrows) {
			printf("   +");
			for (i=0; i<size_x; i++) {
				printf("---");
			}
			printf("> x\n     ");
			for (i=0; i<size_x; i++) {
				printf("%2u ",i);
			}
			printf("\n");
		}
	}
}

void displayGamePrompt(int bump) {
	if (bump) {
		printf("<ouch>");
	} else {
		printf("==>");
	}
	fflush( stdout );
}

void displayGameHelp(void) {
	printf("8 : up / 6 : right / 2 : down / 4 : left / 'q' : quit\n");
}

void displayQuit(void) {
	printf("+---------------+\n");
	printf("| you loose !!! |\n");
	printf("+---------------+\n");
}

void displayEnd(int nbmove,int startx, int starty,int moves[]) {
	int replaymax = (nbmove < 1024)?nbmove:1024;
	int cpt;
	int x = startx;
	int y = starty;

	for (cpt = 0; cpt < replaymax; cpt++) {
		if (!(model[x][y] & REPLAY_SPOT)) {
			model[x][y] += REPLAY_SPOT;
		}
		displayMatrixAndPos(x,y,0,'o');
		fflush( stdout );
		sleep(1);
		x += offset_x[moves[cpt]];
		y += offset_y[moves[cpt]];
	}

	printf("+-------------+\n");
	printf("| you win !!! |\n");
	printf("+-------------+\n");
	printf("in %d moves\n",nbmove);
}
